package com.sun.xml.internal.ws.encoding;

import java.nio.charset.Charset;
import java.io.OutputStreamWriter;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.io.InputStreamReader;
import java.io.IOException;
import javax.activation.DataSource;
import java.awt.datatransfer.DataFlavor;
import javax.activation.ActivationDataFlavor;
import javax.activation.DataContentHandler;

public class StringDataContentHandler implements DataContentHandler
{
    private static final ActivationDataFlavor myDF;
    
    public StringDataContentHandler() {
    }
    
    protected ActivationDataFlavor getDF() {
        return StringDataContentHandler.myDF;
    }
    
    public DataFlavor[] getTransferDataFlavors() {
        return new DataFlavor[] { this.getDF() };
    }
    
    public Object getTransferData(DataFlavor df, DataSource ds) throws IOException {
        if (this.getDF().equals(df)) {
            return this.getContent(ds);
        }
        return null;
    }
    
    public Object getContent(DataSource ds) throws IOException {
        String enc = null;
        InputStreamReader is;
        try {
            enc = this.getCharset(ds.getContentType());
            is = new InputStreamReader(ds.getInputStream(), enc);
        }
        catch (IllegalArgumentException iex) {
            throw new UnsupportedEncodingException(enc);
        }
        try {
            int pos = 0;
            char[] buf = new char[1024];
            int count;
            while ((count = is.read(buf, pos, buf.length - pos)) != -1) {
                pos += count;
                if (pos < buf.length) {
                    continue;
                }
                int size = buf.length;
                if (size < 262144) {
                    size += size;
                }
                else {
                    size += 262144;
                }
                char[] tbuf = new char[size];
                System.arraycopy(buf, 0, tbuf, 0, pos);
                buf = tbuf;
            }
            return new String(buf, 0, pos);
        }
        finally {
            try {
                is.close();
            }
            catch (IOException ex) {}
        }
    }
    
    public void writeTo(Object obj, String type, OutputStream os) throws IOException {
        if (!(obj instanceof String)) {
            throw new IOException("\"" + this.getDF().getMimeType() + "\" DataContentHandler requires String object, was given object of type " + obj.getClass().toString());
        }
        String enc = null;
        OutputStreamWriter osw;
        try {
            enc = this.getCharset(type);
            osw = new OutputStreamWriter(os, enc);
        }
        catch (IllegalArgumentException iex) {
            throw new UnsupportedEncodingException(enc);
        }
        String s = (String)obj;
        osw.write(s, 0, s.length());
        osw.flush();
        return;
    }
    
    private String getCharset(String type) {
        try {
            ContentType ct = new ContentType(type);
            String charset = ct.getParameter("charset");
            if (charset == null) {
                charset = "us-ascii";
            }
            return Charset.forName(charset).name();
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    static {
        myDF = new ActivationDataFlavor(String.class, "text/plain", "Text String");
    }
}
